/**
 * @fileoverview Conditionally rendered text nodes should be wrapped in an element (generally a <span>), otherwise Google Translate will cause a browser error.
 * @author alistair-coup
 */
'use strict';

/** @type {import('eslint').Rule.RuleModule} */
module.exports = {
  meta: {
    type: 'problem',
    docs: {
      description:
        'Conditionally rendered text nodes should be wrapped in an element (generally a <span>), otherwise Google Translate can cause a browser error.',
    },
    schema: [],
    messages: {
      'conditional-text-node':
        'When rendered as a direct child of a JSX element, conditionally rendered text nodes with siblings (elements or nodes) should be wrapped in an element (generally a `<span>`) to prevent Google Translate causing a browser error while manipulating the DOM.',
      'text-node-preceeded-by-conditional':
        'When rendered as a direct child of a JSX element, static text nodes which are preceeded by a conditional expression should be wrapped in an element (generally a `<span>`) to prevent Google Translate causing a browser error while manipulating the DOM.',
    },
  },

  create(context) {
    function isConditionallyRendered(node) {
      return (
        node.parent &&
        (node.parent.type === 'ConditionalExpression' ||
          node.parent.type === 'LogicalExpression')
      );
    }

    function isWithinJSXEspressionContainer(node) {
      return node.parent && node.parent.type === 'JSXExpressionContainer';
    }

    function isWithinJSXElement(node) {
      return node.parent && node.parent.type === 'JSXElement';
    }

    function hasSiblings(node) {
      return (
        node.parent &&
        node.parent.children.length > 1 &&
        node.parent.children.some(
          (child) =>
            child.start !== node.start &&
            !(child.type === 'Literal' && child.value.trim() === '')
        )
      );
    }

    function siblingsDoPreceedNode(node) {
      return (
        node.parent &&
        node.parent.children.some(
          (child) =>
            !(child.type === 'Literal' && child.value.trim() === '') &&
            child.start < node.start
        )
      );
    }

    function functionWasCalled(node) {
      if (!node.parent || node.parent.type === 'ReturnExpression') {
        return false;
      } else if (node.parent.type === 'CallExpression') {
        return true;
      } else {
        return functionWasCalled(node.parent);
      }
    }

    return {
      // conditionally rendered text nodes (string literals, or other literals that are coerced to strings)
      Literal(node) {
        console.log('---------------------------------------- literall', node);
        console.log(
          'isConditionallyRendered(node.parent): ',
          isConditionallyRendered(node.parent)
        );
        console.log(
          'isWithinJSXEspressionContainer(node.parent): ',
          isWithinJSXEspressionContainer(node.parent)
        );
        console.log(
          'isWithinJSXElement(node.parent.parent): ',
          isWithinJSXElement(node.parent.parent)
        );
        console.log(
          'hasSiblings(node.parent.parent): ',
          hasSiblings(node.parent.parent)
        );
        if (
          isConditionallyRendered(node) &&
          isWithinJSXEspressionContainer(node.parent) &&
          isWithinJSXElement(node.parent.parent) &&
          hasSiblings(node.parent.parent)
        ) {
          context.report({
            node,
            messageId: 'conditional-text-node',
          });
        }
      },
      // conditionally rendered text nodes generated by calling toLocaleString
      Identifier(node) {
        if (node.name === 'toLocaleString' && functionWasCalled(node)) {
          const callExpression = node.parent.parent;
          if (
            isConditionallyRendered(callExpression) &&
            isWithinJSXEspressionContainer(callExpression.parent) &&
            isWithinJSXElement(callExpression.parent.parent) &&
            hasSiblings(callExpression.parent.parent)
          ) {
            context.report({
              node: callExpression,
              messageId: 'conditional-text-node',
            });
          }
        }
      },
      // static text nodes preceeded by a conditional expression
      JSXText(node) {
        if (hasSiblings(node) && siblingsDoPreceedNode(node)) {
          context.report({
            node,
            messageId: 'text-node-preceeded-by-conditional',
          });
        }
      },
    };
  },
};
